/*
 * mql_result.h
 *
 * MQL results
 *
 * Ulrik Petersen
 * Created: 3/8-2001
 * Last update: 3/30-2005
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2005  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation, license version 2.  
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *   02111-1307 USA
 *
 *
 *   Special exception
 *   =================
 * 
 *   In addition, as a special exception, Ulrik Petersen, the
 *   copyright holder of Emdros, gives permission to link Emdros, in
 *   whole or in part, with the libraries which are normally
 *   distributed with:
 *   
 *   a) Sun's Java platform,
 *   b) Python, 
 *   c) Jython,
 *   d) Ruby, and/or 
 *   e) Perl 
 *   f) PostgreSQL
 *   g) OpenSSL
 *
 *   (or with modified versions of these), and to distribute linked
 *   combinations including both Emdros, in whole or in part, and one
 *   or more of the libraries normally distributed with (a)-(g) above.
 *
 *   Please note: This gives you special rights concerning the
 *   libraries which normally accompany the above pieces of software.
 *   It gives you no special rights concerning software that you write
 *   yourself.  You must obey the GNU General Public License in all
 *   respects for all of the code used other than the libraries
 *   normally distributed with (a)-(g) above.
 *
 *   If you modify this file, you may extend this exception to your
 *   version of the file, but you are not obligated to do so. If you
 *   do not wish to do so, delete this exception statement from your
 *   version.
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 *
 *   Website
 *   =======
 *
 *   Emdros has a website here:
 *
 *   http://emdros.org
 *
 *
 *
 **************************************************************************/

/**@file mql_result.h
 *@brief Header file for MQLResult (MQL layer)
 */



#ifndef MQL_RESULT__H__
#define MQL_RESULT__H__

#include <emdf.h>
#include <table.h>
#include <string>
#include <map>
#include <list>

class MQLExecEnv; // forward declaration
class Sheaf; // forward declaration
class EMdFOutput; // forward declaration
class FlatSheaf; // forward declaration
class EMdFDB; // forward declaration
class StringList; // forward declaration


class MQLResult : public Table {
private:
  Sheaf *m_sheaf;
  FlatSheaf *m_flatSheaf;
  bool m_bDeleteFullSheaf;
  bool m_bDeleteFlatSheaf;
public:
#ifndef SWIG
  MQLResult(); // For lists
  MQLResult(Sheaf* sheaf, MQLExecEnv *pEE);
  void setDeleteFullSheaf(bool bDoDelete);
  void setDeleteFlatSheaf(bool bDoDelete);
#endif // !defined SWIG
  ~MQLResult();
  bool isSheaf() const;
  bool isFlatSheaf() const;
  bool isTable() const;
  Sheaf* getSheaf(void) { return m_sheaf; };
  FlatSheaf *getFlatSheaf(void) { return m_flatSheaf; };
  Table* getTable(void) { return this; };
  void out(EMdFOutput *pOut) const;
  static void printDTD(EMdFOutput* pOut);
#ifndef SWIG
  bool flatten(const std::list<std::pair<id_d_t, std::string> >& object_types);
#endif
  bool flatten(); // For all object types in the sheaf
  bool flatten(StringList *pObjectTypeNames, EMdFDB *pDB);
};


#endif /* MQL_RESULT__H__ */
