/*
 * templatelang.h
 *
 * Template language classes
 *
 * Ulrik Sandborg-Petersen
 * Created: 7/28-2008
 * Last update: 3/9-2009
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2008-2009  Ulrik Sandborg-Petersen
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation, license version 2.  
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *   02111-1307 USA
 *
 *
 *   Special exception
 *   =================
 * 
 *   In addition, as a special exception, Ulrik Petersen, the
 *   copyright holder of Emdros, gives permission to link Emdros, in
 *   whole or in part, with the libraries which are normally
 *   distributed with:
 *   
 *   a) Sun's Java platform,
 *   b) Python, 
 *   c) Jython,
 *   d) Ruby, and/or 
 *   e) Perl 
 *   f) PostgreSQL
 *   g) OpenSSL
 *
 *   (or with modified versions of these), and to distribute linked
 *   combinations including both Emdros, in whole or in part, and one
 *   or more of the libraries normally distributed with (a)-(g) above.
 *
 *   Please note: This gives you special rights concerning the
 *   libraries which normally accompany the above pieces of software.
 *   It gives you no special rights concerning software that you write
 *   yourself.  You must obey the GNU General Public License in all
 *   respects for all of the code used other than the libraries
 *   normally distributed with (a)-(g) above.
 *
 *   If you modify this file, you may extend this exception to your
 *   version of the file, but you are not obligated to do so. If you
 *   do not wish to do so, delete this exception statement from your
 *   version.
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 *
 *   Website
 *   =======
 *
 *   Emdros has a website here:
 *
 *   http://emdros.org
 *
 *
 *
 **************************************************************************/

#ifndef TEMPLATELANG_HEADER_H_
#define TEMPLATELANG_HEADER_H_

#include <string>
#include <map>
#include <list>

#include "templatelang_lexer.h"
#include <qdxml.h>

class TemplateLangExecEnv; // Forward declaration
class MemObject; // Forward declaration

class TemplateASTNode {
 protected:
	TemplateASTNode *m_pNext;
 public:
	TemplateASTNode();
	virtual ~TemplateASTNode();
	
	virtual void setNext(TemplateASTNode *pNext) { m_pNext = pNext; };
	virtual TemplateASTNode *getNext(void) { return m_pNext; };

	virtual void exec(TemplateLangExecEnv *pEE);
};

typedef enum {
	kTemplateID_D,
	kTemplateFirstMonad,
	kTemplateDBName
} eTemplateSimpleNodeKind;


class TemplateSimpleNode : public TemplateASTNode {
 protected:
	eTemplateSimpleNodeKind m_kind;
 public:
	TemplateSimpleNode(eTemplateSimpleNodeKind kind);
	virtual ~TemplateSimpleNode();

	virtual void exec(TemplateLangExecEnv *pEE);
};

class TemplateFeature : public TemplateASTNode {
 protected:
	long m_index;
	bool m_bMangleFeature;
 public:
	TemplateFeature(long index, bool bMangleFeature);
	virtual ~TemplateFeature();

	virtual void exec(TemplateLangExecEnv *pEE);
};

class TemplateAttrib : public TemplateASTNode {
 protected:
	std::string m_attrib_name;
 public:
	TemplateAttrib(std::string *pString);
	virtual ~TemplateAttrib();

	virtual void exec(TemplateLangExecEnv *pEE);
};

class TemplateCounter : public TemplateASTNode {
 protected:
	std::string m_counter_name;
 public:
	TemplateCounter(std::string *pString);
	virtual ~TemplateCounter();

	virtual void exec(TemplateLangExecEnv *pEE);
};

class TemplateIncCounter : public TemplateASTNode {
 protected:
	std::string m_counter_name;
 public:
	TemplateIncCounter(std::string *pString);
	virtual ~TemplateIncCounter();

	virtual void exec(TemplateLangExecEnv *pEE);
};

class TemplateString : public TemplateASTNode {
 protected:
	std::string m_string;
 public:
	TemplateString(std::string *pString);
	virtual ~TemplateString();

	virtual void exec(TemplateLangExecEnv *pEE);
};


class TemplateIf : public TemplateASTNode {
 protected:
	TemplateASTNode *m_pCondition;
	TemplateASTNode *m_pExecIfTrue;
	TemplateASTNode *m_pExecIfFalse;
 public:
	TemplateIf(TemplateASTNode *pCondition,
		   TemplateASTNode *pExecIfTrue,
		   TemplateASTNode *pExecIfFalse);
	virtual ~TemplateIf();

	virtual void exec(TemplateLangExecEnv *pEE);
};

class TemplateIfFeatureEqual : public TemplateASTNode {
 protected:
	long m_feature_index;
	std::string m_value;
 public:
	TemplateIfFeatureEqual(long feature_index, std::string *pString);
	virtual ~TemplateIfFeatureEqual();

	virtual void exec(TemplateLangExecEnv *pEE);
};


class TemplateIfAttribEqual : public TemplateASTNode {
 protected:
	std::string m_attrib_name;
	std::string m_value;
 public:
	TemplateIfAttribEqual(std::string *pAttribName, std::string *pString);
	virtual ~TemplateIfAttribEqual();

	virtual void exec(TemplateLangExecEnv *pEE);
};


class TemplateIfHasAttrib : public TemplateASTNode {
 protected:
	std::string m_attrib_name;
 public:
	TemplateIfHasAttrib(std::string *pAttribName);
	virtual ~TemplateIfHasAttrib();

	virtual void exec(TemplateLangExecEnv *pEE);
};


class TemplateListAppend : public TemplateASTNode {
 protected:
	std::string m_strListID;
	TemplateASTNode *m_pContents;
 public:
	TemplateListAppend(std::string *pStrListID, TemplateASTNode *pContents);
	virtual ~TemplateListAppend();

	virtual void exec(TemplateLangExecEnv *pEE);
};


class TemplateListEmit : public TemplateASTNode {
 protected:
	std::string m_strListID;
	std::string m_strSeparator;
 public:
	TemplateListEmit(std::string *pStrListID, std::string *pStrSeparator);
	virtual ~TemplateListEmit();

	virtual void exec(TemplateLangExecEnv *pEE);
};



class TemplateLangExecEnv {
 protected:
	std::string *m_pError;
	TemplateASTNode *m_pNode;
	std::string *m_pOutput;
	std::string m_DBName;
	AttributeMap m_attrs;
	typedef std::map<std::string, long> CounterMap;
	CounterMap m_counters;
	typedef std::map<std::string, std::list<std::string> > StringListMap;
	StringListMap m_lists;
	std::string m_strCurrentList;
	bool m_bIsAddingToList;
 public:
	MemObject *m_pObject;
	TemplateLanguageScanner *pLexer;

	TemplateLangExecEnv();
	~TemplateLangExecEnv();
	bool bLastIfCondition;
	bool bSyntaxError;

	void setObject(MemObject *pObj) { m_pObject = pObj; };

	std::string getDBName() const { return m_DBName; };
	void setDBName(const std::string& DBName) { m_DBName = DBName; };

	void setTopNode(TemplateASTNode *pNode);
	TemplateASTNode *extractTopNode(void);

	void clearError(void);
	std::string getError();
	void addToError(const std::string& addition);
	void prependError(const std::string& addition);

	void clearOutput(void);
	std::string getOutput();
	std::string getAndClearOutput();
	void addToOutput(const std::string& addition);

	void setAttributes(const AttributeMap& attrs);
	bool hasAttribute(const std::string& attrib_name);
	std::string getAttribute(const std::string& attrib_name);

	long getCounter(const std::string& counter_name); // Initializes to 0 if not present!
	void incCounter(const std::string& counter_name); // Initializes to 0 if not present!

	void enterListMode(const std::string& list_name);
	void exitListMode();
	void emitList(const std::string& list_name, const std::string& separator);

	void clear();

	void clean();
};

extern TemplateASTNode *parseTemplate(TemplateLangExecEnv* pEE, const std::string& template_string, bool& bResult);

extern std::string escapeXMLEntities(const std::string& input);

#endif
